vigilance_fn_default_inf = function(pattern, category_w, rho)
{
  return(inf_norm(pattern_compare(pattern, category_w)>=(rho*inf_norm(pattern))))
}
#' Checks whether the vigilance test was passed (using 1-norm)
#'
#' @param pattern input pattern
#' @param category_w weights of selected cluster
#' @param rho vigilance parameter
#'
#' @return Boolean specifying whether the test was passed or not.
vigilance_fn_default_one = function(pattern, category_w, rho)
{
  return(one_norm(pattern_compare(pattern, category_w)>=(rho*inf_norm(pattern))))
}

#' Matches a pattern and category. This is the default function used. You may replace it with a custom one that takes the same inputs and produces a scalar output.
#'
#' @param pattern Current input pattern.
#' @param category_w Weight for the current category being matched.
#' @param alpha Choice parameter alpha > 0. This is specified in \code{\link[FuzzyART:FuzzyART_train]{FuzzyART_train()}}.
#'
#' @return The fuzzy extension of the logical AND statement.
#' @export
match_fn_default = function(pattern, category_w, alpha)
{
  return(one_norm(pattern_compare(pattern, category_w)/(alpha+one_norm(pattern))))
}
#' update_fn_default
#'
#' @param pattern The input observation.
#' @param category_w Current category weight.
#' @param beta Learnign Rate. This is specified in \code{\link[FuzzyART:FuzzyART_train]{FuzzyART_train()}}.
#' @param distance_fn A distance function with the parameters and outputs like \code{\link[FuzzyART:fuzzy_and]{fuzzy_and()}}.
#'
#' @export
update_fn_default = function(pattern, category_w, beta, distance_fn= fuzzy_and)
{
  return(beta*pattern_compare(pattern,category_w,distance_fn = distance_fn)+(1-beta)*category_w)
}
#' Fuzzy And Function
#'
#' @param a First input vector
#' @param b Second input vector. Should be of the same dimension as a.
#'
#' @return Returns the Fuzzy AND of two respective values in a vector. See the example for details.
#' @export
#'
#' @examples fuzzy_and(1,0) # = 0
#' fuzzy_and(0, 1) # = 0
#' fuzzy_and(0, 0) # = 0
#' fuzzy_and(1, 1) # = 1
#' fuzzy_and(c(0.5, 1), c(0.25,0)) # = c(0.25,0)
fuzzy_and = function(a,b)
{
  comb = cbind(a,b)
  return(apply(X = comb,FUN = min,MARGIN = 1))
}

#' Computes the inf-norm of a vector (not compatiple with matrices).
#'
#' @param pattern The input pattern (vector) for the computation of the norm.
#'
#' @return Returns the maximum norm of a vector.
#' @export
#'
#' @examples a = c(-3,2,-6,5)
#' inf_norm(a) #=6
inf_norm = function(pattern)
{
  return(max(abs(pattern)))
}

#' Computes the 1-norm of a vector (not compatiple with matrices)
#'
#' @param pattern The input pattern (vector) for the computation of the norm.
#'
#' @return The 1-norm of the input pattern.
#' @export
#'
#' @examples a = c(-3,2,-6,5)
#' one_norm(a) #=16
one_norm = function(pattern)
{
  return(sum(abs(pattern)))
}
