% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_fcts.R
\name{FuzzyART_train}
\alias{FuzzyART_train}
\title{Specifies and trains a FuzzyART model}
\usage{
FuzzyART_train(
  inputs,
  rho,
  alpha,
  beta,
  max_epochs,
  shuffle = TRUE,
  random_seed = NA,
  w_init = NA,
  distance_fn = fuzzy_and,
  match_fn = match_fn_default,
  vigilance_fn = vigilance_fn_default_one,
  update_fn = update_fn_default,
  max_clusters = 1000,
  eps = 10^-6,
  show_status = TRUE,
  beta_decay = 1
)
}
\arguments{
\item{inputs}{Specifies input data as a data.frame, matrix or simiar structure. Inputs should be normalized prior to running by using the \code{\link[=normalize]{normalize()}} function.}

\item{rho}{Vigilance parameter in (0,1).}

\item{alpha}{Choice parameter alpha > 0. Can be viewed as a regularization parameter penalizing large weights.}

\item{beta}{Learning rate in (0,1).}

\item{max_epochs}{Maximum number of iterations over the input.}

\item{shuffle}{TRUE allows the input to be shuffled during training. FALSE disables this.}

\item{random_seed}{A number specifying the seed for the random number generator while shuffling.}

\item{w_init}{Initial model weights.}

\item{distance_fn}{A distance function to determine how close an observation is to the category weight (see \code{\link[=fuzzy_and]{fuzzy_and()}} for specifications).}

\item{match_fn}{A match function like \code{\link[FuzzyART:match_fn_default]{match_fn_default()}}.}

\item{vigilance_fn}{A vigilance function like \code{\link[FuzzyART:vigilance_fn_default_one]{vigilance_fn_default_one()}}.}

\item{update_fn}{An update function like \code{\link[FuzzyART:update_fn_default]{update_fn_default()}}.}

\item{max_clusters}{The maximum numbers of cluster allowed.}

\item{eps}{Tolerance. Default is 10^-6.}

\item{show_status}{TRUE (default) will display the progress of learning measured as a percentage of max_epochs. FALSE will supress the status.}

\item{beta_decay}{A decay factor between 0 and 1. The closer to 0, the faster the decay.}
}
\value{
A list specifying the trained parameters, the labels for the input dazta, and the parameters used
}
\description{
Specifies and trains a FuzzyART model
}
\examples{
normalized_inputs = normalize(df = subset(iris,select = -Species))
mod  = FuzzyART_train(normalized_inputs,alpha = .8,rho = .5,beta = .08,
                      max_epochs = 1000,max_clusters =20,eps = 10^-8,random_seed = 4)
mod$Labels
plot(iris, col = mod$Labels)
}
\references{
Carpenter, Gail A., Stephen Grossberg, and David B. Rosen. "Fuzzy ART: Fast stable learning and categorization of analog patterns by an adaptive resonance system." Neural networks 4.6 (1991): 759-771.
}
